/***
 * Excerpted from "Build Talking Apps",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/cwalexa for more book information.
***/
const nock = require('nock');

module.exports = {
  onRequest: (test, requestEnvelope) => {
    const destination = test.testSuite.configuration.destination;
    const departureDate = test.testSuite.configuration.departureDate;
    const returnDate = test.testSuite.configuration.returnDate;
    const dialogState  = test.testSuite.configuration.dialogState;

    if (destination) {
      const resolvedDestinationSlot = JSON.parse(JSON.stringify(mockResolvedDestination));
      if (destination == destination.toLowerCase()) {
        resolvedDestinationSlot.resolutions.resolutionsPerAuthority[0].status.code = 'ER_NO_MATCH';
        resolvedDestinationSlot.resolutions.resolutionsPerAuthority[0].values = [];
        resolvedDestinationSlot.value = destination;
        mockRequest.intent.slots.destination = resolvedDestinationSlot;
      } else {
        resolvedDestinationSlot.value = destination.toLowerCase();
        resolvedDestinationSlot.resolutions.resolutionsPerAuthority[0].values[0].value.name = destination;
        resolvedDestinationSlot.resolutions.resolutionsPerAuthority[0].values[0].value.id = destination.toUpperCase();
        mockRequest.intent.slots.destination = resolvedDestinationSlot;
      }
    }

    if (departureDate) {
        mockRequest.intent.slots.departureDate.value = departureDate.toString();
        mockRequest.intent.slots.departureDate.source = 'USER';
    }

    if (returnDate) {
        mockRequest.intent.slots.returnDate.value = returnDate.toString();
        mockRequest.intent.slots.returnDate.source = 'USER';
    }

    if (dialogState) {
        mockRequest.dialogState = dialogState.toString();
        if (dialogState != 'STARTED') {
          requestEnvelope.context.System.user.accessToken = 'TEST_GOOGLE_ACCESS_TOKEN';
        }
        if (dialogState == 'COMPLETED') {
          mockPostData.summary = "Trip to " + destination;
          mockPostData.description = "Trip to " + destination;
          mockPostData.start.date = departureDate.toString();
          mockPostData.end.date = returnDate.toString();
          nock('https://www.googleapis.com')
              .post('/calendar/v3/calendars/primary/events', mockPostData)
              .reply(200, googleCalendarResponse);
        }
    }

    requestEnvelope.request = mockRequest;
  }
};

const mockRequest = {
    "type": "IntentRequest",
    "requestId": "amzn1.echo-api.request.0fc3de0e-56b0-4825-b3df-77efb25fac25",
    "timestamp": "2019-09-12T23:47:58Z",
    "locale": "en-US",
    "intent": {
      "name": "ScheduleTripIntent",
      "confirmationStatus": "NONE",
      "slots": {
        "destination": {
          "name": "destination",
          "confirmationStatus": "NONE"
        },
        "returnDate": {
          "name": "returnDate",
          "confirmationStatus": "NONE"
        },
        "departureDate": {
          "name": "departureDate",
          "confirmationStatus": "NONE"
        }
      }
    },
    "dialogState": "IN_PROGRESS"
  };

  const mockResolvedDestination = {
    "name": "destination",
    "value": "value",
    "resolutions": {
      "resolutionsPerAuthority": [
        {
          "authority": "amzn1.er-authority.echo-sdk.amzn1.ask.skill.SKILL_ID.PLANETS",
          "status": {
            "code": "ER_SUCCESS_MATCH"
          },
          "values": [
            {
              "value": {
                "name": "Name",
                "id": "ID"
              }
            }
          ]
        }
      ]
    },
    "confirmationStatus": "NONE",
    "source": "USER"
  };

  const mockPostData = {
    "end": {
     "date": "2019-10-09"
    },
    "start": {
     "date": "2019-10-05"
    },
    "description": "Trip to mars",
    "summary": "Trip to mars"
   };

  const googleCalendarResponse = {
    "id": "fakeeventid"
   };
   