/***
 * Excerpted from "Build Talking Apps",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/cwalexa for more book information.
***/
const Alexa = require('ask-sdk-core');
const TripSaver = require('./GoogleCalendarTripSaver');
const getResolvedSlotValue = require('./Helpers');
const toFriendlyDate = require('./dateUtils');

const ScheduleTripIntentHandler = {
  canHandle(handlerInput) {
    return Alexa.getRequestType(
              handlerInput.requestEnvelope) === 'IntentRequest'
      && Alexa.getIntentName(
              handlerInput.requestEnvelope) === 'ScheduleTripIntent'
      && Alexa.getDialogState(handlerInput.requestEnvelope) === 'COMPLETED';
  },
  async handle(handlerInput) {
    const destination =
        getResolvedSlotValue(handlerInput.requestEnvelope, 'destination');
    const departureDate =
        Alexa.getSlotValue(handlerInput.requestEnvelope, 'departureDate');
    const returnDate =
        Alexa.getSlotValue(handlerInput.requestEnvelope, 'returnDate');

    await TripSaver.saveTrip(
        handlerInput, destination, departureDate, returnDate);

    const speakOutput = handlerInput.t('SCHEDULED_MSG',
        { destination: destination });

    const cardTitle = handlerInput.t('SCHEDULED_CARD_TITLE');
    const cardContent = handlerInput.t('SCHEDULED_CARD_MSG',
    {
      destination: destination,
      departureDate: departureDate,
      returnDate: returnDate
    });
    const baseUrl = 'https://starport75.dev/images/planets/';
    const smallImageUrl = baseUrl + destination.toLowerCase() + '_sm.jpg';
    const largeImageUrl = baseUrl + destination.toLowerCase() + '_lg.jpg';

    if (Alexa.getSupportedInterfaces(handlerInput.requestEnvelope)
                                            ['Alexa.Presentation.APL']) {
      const locale = Alexa.getLocale(handlerInput.requestEnvelope);
      handlerInput.responseBuilder
          .addDirective({
            type : 'Alexa.Presentation.APL.RenderDocument',
            document: require('./apl/tripComplete.json'),
/*
            document: require('./apl/tripComplete.json')
*/
            datasources: {
              "tripData": {
                "titleText": handlerInput.t('ENJOY_YOUR_TRIP_TITLE'),
                "destination": destination,
                "departureLabel": handlerInput.t('DEPARTURE_LABEL'),
                "departureDate": toFriendlyDate(departureDate, locale),
                "returnLabel": handlerInput.t('RETURNING_LABEL'),
                "returnDate": toFriendlyDate(returnDate, locale)
              }
            }
        });
    }

    return handlerInput.responseBuilder
      .speak(speakOutput)
      .withStandardCard(cardTitle, cardContent, smallImageUrl, largeImageUrl)
      .withShouldEndSession(true)
      .getResponse();
  },
};

module.exports=ScheduleTripIntentHandler;
