/***
 * Excerpted from "Build Talking Apps",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/cwalexa for more book information.
***/
const Alexa = require('ask-sdk-core');
const reminderSender = require ('./reminderSender');

const YesIntentHandler = {
  canHandle(handlerInput) {
    const requestEnvelope = handlerInput.requestEnvelope;
    const sessionAttributes =
        handlerInput.attributesManager.getSessionAttributes();

    return Alexa.getRequestType(requestEnvelope) === 'IntentRequest'
        && Alexa.getIntentName(requestEnvelope) === 'AMAZON.YesIntent'
        && sessionAttributes.questionAsked === 'SetAReminder';
  },
  async handle(handlerInput) {
    const sessionAttributes =
        handlerInput.attributesManager.getSessionAttributes();
    sessionAttributes.questionAsked = null;
    const destination = sessionAttributes.destination;
    const departureDate = sessionAttributes.departureDate;
    handlerInput.attributesManager.setSessionAttributes(sessionAttributes);

    var speakOutput = handlerInput.t('YES_REMINDER');

    try {
      await reminderSender.setReminder(
                handlerInput, destination, departureDate);
    } catch (error) {
      speakOutput = error.message;
    }

    return handlerInput.responseBuilder
        .speak(speakOutput)
        .withShouldEndSession(true)
        .getResponse();
  }
};

const NoIntentHandler = {
  canHandle(handlerInput) {
    const requestEnvelope = handlerInput.requestEnvelope;
    const sessionAttributes =
        handlerInput.attributesManager.getSessionAttributes();

    return Alexa.getRequestType(requestEnvelope) === 'IntentRequest'
        && Alexa.getIntentName(requestEnvelope) === 'AMAZON.NoIntent'
        && sessionAttributes.questionAsked === 'SetAReminder';
  },
  handle(handlerInput) {
    const sessionAttributes =
        handlerInput.attributesManager.getSessionAttributes();
    sessionAttributes.questionAsked = null;
    handlerInput.attributesManager.setSessionAttributes(sessionAttributes);

    const speakOutput = handlerInput.t('NO_REMINDER');

    return handlerInput.responseBuilder
        .speak(speakOutput)
        .withShouldEndSession(true)
        .getResponse();
  }
};

module.exports = {
  YesIntentHandler: YesIntentHandler,
  NoIntentHandler: NoIntentHandler
};
