/***
 * Excerpted from "Build Talking Apps",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/cwalexa for more book information.
***/
/* *
 * This sample demonstrates handling intents from an Alexa skill using the Alexa Skills Kit SDK (v2).
 * Please visit https://alexa.design/cookbook for additional examples on implementing slots, dialog management,
 * session persistence, api calls, and more.
 * */

const Alexa = require('ask-sdk-core');
const i18n = require('i18next');
const languageStrings = require('./languageStrings');

const HelloWorldIntentHandler = require('./HelloWorldIntentHandler');
const ScheduleTripIntentHandler = require('./ScheduleTripIntentHandler');
const ScheduleTripIntentHandler_InProgress =
              require('./ScheduleTripIntentHandler_InProgress');
const StandardHandlers = require('./StandardHandlers');

// This request interceptor will bind a translation function 't' to the handlerInput
const LocalisationRequestInterceptor = {
    process(handlerInput) {
        i18n.init({
            lng: Alexa.getLocale(handlerInput.requestEnvelope),
            resources: languageStrings
        }).then((t) => {
            handlerInput.t = (...args) => t(...args);
        });
    }
};


/**
 * This handler acts as the entry point for your skill, routing all request and response
 * payloads to the handlers above. Make sure any new handlers or interceptors you've
 * defined are included below. The order matters - they're processed top to bottom
 * */
exports.handler = Alexa.SkillBuilders.custom()
    .addRequestHandlers(
/*
    ...
*/
        HelloWorldIntentHandler,
        ScheduleTripIntentHandler,
        ScheduleTripIntentHandler_InProgress,
        StandardHandlers.LaunchRequestHandler,
        StandardHandlers.HelpIntentHandler,
        StandardHandlers.CancelAndStopIntentHandler,
        StandardHandlers.FallbackIntentHandler,
        StandardHandlers.SessionEndedRequestHandler,
        StandardHandlers.IntentReflectorHandler
        )
    .addErrorHandlers(
        StandardHandlers.ErrorHandler)
    .withApiClient(new Alexa.DefaultApiClient())
    .addRequestInterceptors(
        LocalisationRequestInterceptor)
    .withCustomUserAgent('sample/hello-world/v1.2')
    .lambda();
