/***
 * Excerpted from "Build Talking Apps",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/cwalexa for more book information.
***/
const Alexa = require('ask-sdk-core');
const ProductUtils = require('./productUtils');
const StandardHandlers = require('./StandardHandlers');

const BuyIntentHandler = {
  canHandle(handlerInput) {
    const requestEnvelope = handlerInput.requestEnvelope;
    return Alexa.getRequestType(requestEnvelope) === 'IntentRequest'
        && Alexa.getIntentName(requestEnvelope) === 'BuyIntent';
  },
  async handle(handlerInput) {
    const slot = Alexa.getSlot(handlerInput.requestEnvelope, 'productName');
    const referenceName = 
        slot.resolutions.resolutionsPerAuthority[0].values[0].value.id;
    const productId = 
        (await ProductUtils.getProduct(handlerInput, referenceName))
        .productId;

    const rsp = handlerInput.responseBuilder
        .addDirective({
          type: "Connections.SendRequest",
          name: "Buy",
          payload: {
            InSkillProduct: {
              productId: productId
            }
          },
          token: "correlationToken"
        })
        .getResponse();
    return rsp;
  }
};

const RefundIntentHandler = {
  canHandle(handlerInput) {
    const requestEnvelope = handlerInput.requestEnvelope;
    return Alexa.getRequestType(requestEnvelope) === 'IntentRequest'
        && Alexa.getIntentName(requestEnvelope) === 'RefundIntent';
  },
  async handle(handlerInput) {
    const slot = Alexa.getSlot(handlerInput.requestEnvelope, 'productName');
    const referenceName = 
        slot.resolutions.resolutionsPerAuthority[0].values[0].value.id;
    const productId = 
        (await ProductUtils.getProduct(handlerInput, referenceName))
        .productId;

    return handlerInput.responseBuilder
        .addDirective({
            type: "Connections.SendRequest",
            name: "Cancel",
            payload: {
                InSkillProduct: {
                    productId: productId
                }
            },
            token: "correlationToken"
        })
        .getResponse();
  }
};

const ContinueAfterRefundHandler = {
  canHandle(handlerInput) {
    const requestEnvelope = handlerInput.requestEnvelope;
    return Alexa.getRequestType(requestEnvelope) === 'Connections.Response'
        && requestEnvelope.request.payload.purchaseResult === 'ACCEPTED'
        && requestEnvelope.request.name === 'Cancel';
  },
  handle(handlerInput) {
    return StandardHandlers.CancelAndStopIntentHandler.handle(handlerInput);
  }
};

const ContinueAfterPurchaseHandler = {
  canHandle(handlerInput) {
    const requestEnvelope = handlerInput.requestEnvelope;
    return Alexa.getRequestType(requestEnvelope) === 'Connections.Response';
  },
  handle(handlerInput) {
    const requestEnvelope = handlerInput.requestEnvelope;
    const purchaseResult = requestEnvelope.request.payload.purchaseResult;
    return handlerInput.responseBuilder
      .speak(handlerInput.t('PURCHASE_RESPONSE_' + purchaseResult))
      .getResponse();
  }
};

module.exports = {
  BuyIntentHandler: BuyIntentHandler,
  ContinueAfterRefundHandler: ContinueAfterRefundHandler,
  RefundIntentHandler: RefundIntentHandler,
  ContinueAfterPurchaseHandler: ContinueAfterPurchaseHandler
};
