/***
 * Excerpted from "Build Talking Apps",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/cwalexa for more book information.
***/
const Alexa = require('ask-sdk-core');
const ProductUtils = require('./productUtils');

async function validateTravelDates(handlerInput) {
  const departureString =
        Alexa.getSlotValue(handlerInput.requestEnvelope, 'departureDate');
  const returnString =
        Alexa.getSlotValue(handlerInput.requestEnvelope, 'returnDate');

  if (departureString && returnString) {
    const departureDate = new Date(departureString);
    const returnDate = new Date(returnString);
    if (departureDate >= returnDate) {
      throw new Error("Star Port Seventy Five specializes in space travel, " +
      "not time travel. Please specify a return date that is " +
      "after the departure date.");
    }
  }
}

async function validateMoonsDestination(handlerInput) {
  const destinationSlot =
        Alexa.getSlot(handlerInput.requestEnvelope, 'destination');
  if (destinationSlot.resolutions) {
    const resolution =
          destinationSlot.resolutions.resolutionsPerAuthority[0];
    const destination = resolution.status.code === 'ER_SUCCESS_MATCH' ?
          resolution.values[0].value.id :
          destinationSlot.value;
    const destinationName = resolution.status.code === 'ER_SUCCESS_MATCH' ?
          resolution.values[0].value.name :
          destinationSlot.value;

    if (destination && ['LUNA','IO','EUROPA','CALLISTO','GANYMEDE','TITAN']
                          .includes(destination)) {
      const product = 
            await ProductUtils.getProduct(handlerInput, 'MAGNIFICENT_MOONS');
      if (!product || product.entitled !== 'ENTITLED') {
        throw {
          message: handlerInput.t('UPSELL_MOONS_PACK', 
                      { destination: destinationName }),
          productId: product.productId
        };
      }
    }
  }    
}

const ScheduleTripIntentHandler_InProgress = {
  canHandle(handlerInput) {
    return Alexa.getRequestType(
              handlerInput.requestEnvelope) === 'IntentRequest'
      && Alexa.getIntentName(
              handlerInput.requestEnvelope) === 'ScheduleTripIntent'
      && Alexa.getDialogState(handlerInput.requestEnvelope) !== 'COMPLETED';
  },
  async handle(handlerInput) {
    const currentIntent = handlerInput.requestEnvelope.request.intent;

    try {
      await validateMoonsDestination(handlerInput);
    } catch (error) {
      console.error(error);
      return handlerInput.responseBuilder
          .addDirective({
              type: "Connections.SendRequest",
              name: "Upsell",
              payload: {
                  InSkillProduct: {
                    productId: error.productId,
                  },
                  upsellMessage: error.message,
              },
              token: "correlationToken",
          })
          .getResponse();
    }

    try {
      await validateTravelDates(handlerInput);
    } catch (error) {
      currentIntent.slots['returnDate'].value = null;
      return handlerInput.responseBuilder
        .speak(error.message)
        .addDelegateDirective(currentIntent)
        .getResponse();
    }

    return handlerInput.responseBuilder
      .addDelegateDirective(currentIntent)
      .getResponse();
  },
};

module.exports=ScheduleTripIntentHandler_InProgress;
